'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OrNode = exports.AndNode = exports.ExpNode = undefined;

var _exception = require('../exception');

var _ = require('./');

/**
 * Generic boolean expression node
 */
class ExpNode {
  /**
   * Initialise the node
   * @param {FiqlNode[]} children A list of child nodes to evaulate
   * @param {String} operator The boolean operator (e.g. and, or)
   */
  constructor(operator, children) {
    this.operator = operator;
    this.children = children;
  }

  build() {
    if (!Array.isArray(this.children)) {
      throw new _exception.InvalidExpressionChildError('Expression must have Array child.');
    }
    // Recursively build children and then join using the operator
    return `${this.children.map(child => {
      if (child instanceof _.OpNode || child instanceof ExpNode || child instanceof _.GroupNode) {
        return child.build();
      }
      throw new _exception.InvalidExpressionChildError('Boolean expression child must be operators or (grouped) boolean expression.');
    }).join(this.operator)}`;
  }
}

exports.ExpNode = ExpNode;
class AndNode extends ExpNode {
  constructor(children) {
    super(';', children);
  }
}

exports.AndNode = AndNode;
class OrNode extends ExpNode {
  constructor(children) {
    super(',', children);
  }
}
exports.OrNode = OrNode;