'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NotInNode = exports.InNode = exports.GeNode = exports.GtNode = exports.LeNode = exports.LtNode = exports.NeqNode = exports.EqNode = exports.OpNode = undefined;

var _exception = require('../exception');

var _ = require('./');

/**
 * Generic operator node
 */
class OpNode {
  /**
     * Initialises operator node
     * @param {string} selector The operator selector (left-side)
     * @param {string} operator The operator (==, !=, etc.)
     * @param {FiqlNode} args Argument for operator (right-side)
     */
  constructor(selector, operator, args) {
    this.selector = selector;
    this.operator = operator;
    this.args = args;
  }

  build() {
    if (!(this.selector instanceof _.LeafNode)) {
      throw new _exception.InvalidSelectorError('Selectors must be a LeafNode.');
    }
    if (typeof this.operator !== 'string') {
      throw new _exception.InvalidOperatorError('Operator must be a string.');
    }
    return `${this.selector.build()}${this.operator}${this.args.build()}`;
  }
}

exports.OpNode = OpNode;
class EqNode extends OpNode {
  constructor(selector, args) {
    super(selector, '==', args);
  }
}

exports.EqNode = EqNode;
class NeqNode extends OpNode {
  constructor(selector, args) {
    super(selector, '!=', args);
  }
}

exports.NeqNode = NeqNode;
class LtNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=lt=', args);
  }
}

exports.LtNode = LtNode;
class LeNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=le=', args);
  }
}

exports.LeNode = LeNode;
class GtNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=gt=', args);
  }
}

exports.GtNode = GtNode;
class GeNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=ge=', args);
  }
}

exports.GeNode = GeNode;
class InNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=in=', args);
  }
}

exports.InNode = InNode;
class NotInNode extends OpNode {
  constructor(selector, args) {
    super(selector, '=out=', args);
  }
}
exports.NotInNode = NotInNode;